@@ -1,4 +1,4 @@
-# Copyright 2018 Amazon.com, Inc. or its affiliates. All Rights Reserved.
+# Copyright 2018-2020 Amazon.com, Inc. or its affiliates. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the 'License'). You
 # may not use this file except in compliance with the License. A copy of
@@ -10,6 +10,7 @@
 # distributed on an 'AS IS' BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
 # ANY KIND, either express or implied. See the License for the specific
 # language governing permissions and limitations under the License.
+"""Placeholder docstring"""
 from __future__ import absolute_import
 
 from distutils import util
@@ -29,13 +30,13 @@ from sagemaker_containers import _content_types, _logging, _mapping, _params
 
 logger = _logging.get_logger()
 
-SAGEMAKER_BASE_PATH = os.path.join('/opt', 'ml')  # type: str
-BASE_PATH_ENV = 'SAGEMAKER_BASE_DIR'  # type: str
+SAGEMAKER_BASE_PATH = os.path.join("/opt", "ml")  # type: str
+BASE_PATH_ENV = "SAGEMAKER_BASE_DIR"  # type: str
 
 
 def _write_json(obj, path):  # type: (object, str) -> None
     """Writes a serializeable object as a JSON file"""
-    with open(path, 'w') as f:
+    with open(path, "w") as f:
         json.dump(obj, f)
 
 
@@ -67,28 +68,29 @@ def _set_base_path_env():  # type: () -> None
         (bool): indicating whe
     """
 
-    local_config_dir = os.path.join(os.path.expanduser('~'), 'sagemaker_local', 'jobs',
-                                    str(time.time()), 'opt', 'ml')
+    local_config_dir = os.path.join(
+        os.path.expanduser("~"), "sagemaker_local", "jobs", str(time.time()), "opt", "ml"
+    )
 
-    logger.info('Setting environment variable SAGEMAKER_BASE_DIR as %s .' % local_config_dir)
+    logger.info("Setting environment variable SAGEMAKER_BASE_DIR as %s ." % local_config_dir)
     os.environ[BASE_PATH_ENV] = local_config_dir
 
 
 _is_path_configured = _is_training_path_configured()
 
 if not _is_path_configured:
-    logger.info('Directory /opt/ml does not exist.')
+    logger.info("Directory /opt/ml does not exist.")
     _set_base_path_env()
 
 base_dir = os.environ.get(BASE_PATH_ENV, SAGEMAKER_BASE_PATH)  # type: str
 
-code_dir = os.path.join(base_dir, 'code')
+code_dir = os.path.join(base_dir, "code")
 """str: the path of the user's code directory, e.g., /opt/ml/code/"""
 
-model_dir = os.path.join(base_dir, 'model')  # type: str
+model_dir = os.path.join(base_dir, "model")  # type: str
 """str: the directory where models should be saved, e.g., /opt/ml/model/"""
 
-input_dir = os.path.join(base_dir, 'input')  # type: str
+input_dir = os.path.join(base_dir, "input")  # type: str
 """str: the path of the input directory, e.g. /opt/ml/input/
 
 The input_dir, e.g. /opt/ml/input/, is the directory where SageMaker saves input data
@@ -99,9 +101,9 @@ The input_dir, e.g. /opt/ml/input/, is the directory where SageMaker saves input
             str: the path of the input directory, e.g. /opt/ml/input/
 """
 
-_input_data_dir = os.path.join(input_dir, 'data')  # type: str
+_input_data_dir = os.path.join(input_dir, "data")  # type: str
 
-input_config_dir = os.path.join(input_dir, 'config')  # type: str
+input_config_dir = os.path.join(input_dir, "config")  # type: str
 """str: the path of the input directory, e.g. /opt/ml/input/config/
 
 The directory where standard SageMaker configuration files are located, e.g. /opt/ml/input/config/.
@@ -118,7 +120,7 @@ Returns:
     str: the path of the input directory, e.g. /opt/ml/input/config/
 """
 
-output_dir = os.path.join(base_dir, 'output')  # type: str
+output_dir = os.path.join(base_dir, "output")  # type: str
 """str: the path to the output directory, e.g. /opt/ml/output/.
 
 The directory where training success/failure indications will be written, e.g. /opt/ml/output.
@@ -127,9 +129,9 @@ Returns:
     str: the path to the output directory, e.g. /opt/ml/output/.
 """
 
-output_data_dir = os.path.join(output_dir, 'data')  # type: str
+output_data_dir = os.path.join(output_dir, "data")  # type: str
 
-output_intermediate_dir = os.path.join(output_dir, 'intermediate')  # type: str
+output_intermediate_dir = os.path.join(output_dir, "intermediate")  # type: str
 """str: the path to the intermediate output directory, e.g. /opt/ml/output/intermediate.
 
 The directory special behavior is to move artifacts from the training instance to
@@ -138,10 +140,9 @@ Returns:
     str: the path to the intermediate output directory, e.g. /opt/ml/output/intermediate.
 """
 
-
-HYPERPARAMETERS_FILE = 'hyperparameters.json'  # type: str
-RESOURCE_CONFIG_FILE = 'resourceconfig.json'  # type: str
-INPUT_DATA_CONFIG_FILE = 'inputdataconfig.json'  # type: str
+HYPERPARAMETERS_FILE = "hyperparameters.json"  # type: str
+RESOURCE_CONFIG_FILE = "resourceconfig.json"  # type: str
+INPUT_DATA_CONFIG_FILE = "inputdataconfig.json"  # type: str
 
 hyperparameters_file_dir = os.path.join(input_config_dir, HYPERPARAMETERS_FILE)  # type: str
 input_data_config_file_dir = os.path.join(input_config_dir, INPUT_DATA_CONFIG_FILE)  # type: str
@@ -151,7 +152,7 @@ resource_config_file_dir = os.path.join(input_config_dir, RESOURCE_CONFIG_FILE)
 def _create_training_directories():
     """Creates the directory structure and files necessary for training under the base path
     """
-    logger.info('Creating a new training folder under %s .' % base_dir)
+    logger.info("Creating a new training folder under %s ." % base_dir)
 
     os.makedirs(model_dir)
     os.makedirs(input_config_dir)
@@ -162,10 +163,7 @@ def _create_training_directories():
 
     host_name = socket.gethostname()
 
-    resources_dict = {
-        "current_host": host_name,
-        "hosts":        [host_name]
-    }
+    resources_dict = {"current_host": host_name, "hosts": [host_name]}
     _write_json(resources_dict, resource_config_file_dir)
 
 
@@ -191,7 +189,7 @@ def _read_json(path):  # type: (str) -> dict
     Returns:
         (dict[object, object]): A dictionary representation of the JSON file.
     """
-    with open(path, 'r') as f:
+    with open(path, "r") as f:
         return json.load(f)
 
 
@@ -212,8 +210,12 @@ def read_hyperparameters():  # type: () -> dict
         try:
             v = json.loads(v)
         except (ValueError, TypeError):
-            logger.info("Failed to parse hyperparameter %s value %s to Json.\n"
-                        "Returning the value itself", k, v)
+            logger.info(
+                "Failed to parse hyperparameter %s value %s to Json.\n"
+                "Returning the value itself",
+                k,
+                v,
+            )
 
         deserialized_hps[k] = v
 
@@ -229,11 +231,11 @@ https://docs.aws.amazon.com/sagemaker/latest/dg/your-algorithms-training-algo.ht
     Returns:
         resource_config (dict[string, object]): the contents from /opt/ml/input/config/resourceconfig.json.
                         It has the following keys:
-                            - current_host: The name of the current container on the container network.
-                                For example, 'algo-1'.
-                            -  hosts: The list of names of all containers on the container network,
-                                sorted lexicographically. For example, `['algo-1', 'algo-2', 'algo-3']`
-                                for a three-node cluster.
+                            - current_host: The name of the current container on the container
+                                            network. For example, 'algo-1'.
+                            -  hosts: The list of names of all containers on the container
+                                      network, sorted lexicographically. For example,
+                                      `['algo-1', 'algo-2', 'algo-3']` for a three-node cluster.
     """
     return _read_json(resource_config_file_dir)
 
@@ -290,11 +292,11 @@ def num_gpus():  # type: () -> int
         int: number of gpus available in the current container.
     """
     try:
-        cmd = shlex.split('nvidia-smi --list-gpus')
-        output = subprocess.check_output(cmd).decode('utf-8')
-        return sum([1 for x in output.split('\n') if x.startswith('GPU ')])
+        cmd = shlex.split("nvidia-smi --list-gpus")
+        output = subprocess.check_output(cmd).decode("utf-8")
+        return sum([1 for x in output.split("\n") if x.startswith("GPU ")])
     except (OSError, subprocess.CalledProcessError):
-        logger.info('No GPUs detected (normal if no gpus installed)')
+        logger.info("No GPUs detected (normal if no gpus installed)")
         return 0
 
 
@@ -311,16 +313,18 @@ class _Env(_mapping.MappingMixin):
     """Base Class which provides access to aspects of the environment including
     system characteristics, filesystem locations, environment variables and configuration settings.
 
-    The Env is a read-only snapshot of the container environment. It does not contain any form of state.
-    It is a dictionary like object, allowing any builtin function that works with dictionary.
+    The Env is a read-only snapshot of the container environment. It does not contain any form of
+    state. It is a dictionary like object, allowing any builtin function that works with dictionary.
 
     Attributes:
-            current_host (str): The name of the current container on the container network. For example, 'algo-1'.
+            current_host (str): The name of the current container on the container network. For
+                                example, 'algo-1'.
             module_name (str): The name of the user provided module.
             module_dir (str): The full path location of the user provided module.
     """
 
     def __init__(self):
+        """Placeholder docstring"""
         current_host = os.environ.get(_params.CURRENT_HOST_ENV)
         module_name = os.environ.get(_params.USER_PROGRAM_ENV, None)
         module_dir = os.environ.get(_params.SUBMIT_DIR_ENV, code_dir)
@@ -406,17 +410,18 @@ class _Env(_mapping.MappingMixin):
         Returns:
             str: Module name
         """
-        if program_param and program_param.endswith('.py'):
+        if program_param and program_param.endswith(".py"):
             return program_param[:-3]
         return program_param
 
 
 class TrainingEnv(_Env):
     """Provides access to aspects of the training environment relevant to training jobs, including
-    hyperparameters, system characteristics, filesystem locations, environment variables and configuration settings.
+    hyperparameters, system characteristics, filesystem locations, environment variables and
+    configuration settings.
 
-    The TrainingEnv is a read-only snapshot of the container environment during training. It does not contain any form
-    of state.
+    The TrainingEnv is a read-only snapshot of the container environment during training. It does
+    not contain any form of state.
 
     It is a dictionary like object, allowing any builtin function that works with dictionary.
 
@@ -444,33 +449,37 @@ class TrainingEnv(_Env):
             >>>model.save(os.path.join(model_dir, 'saved_model'))
 
     Attributes:
-        input_dir (str): The input_dir, e.g. /opt/ml/input/, is the directory where SageMaker saves input data
-            and configuration files before and during training. The input data directory has the
-            following subdirectories: config (`input_config_dir`) and data (`_input_data_dir`)
+        input_dir (str): The input_dir, e.g. /opt/ml/input/, is the directory where SageMaker saves
+                         input data and configuration files before and during training. The input
+                         data directory has the following subdirectories:
+                         config (`input_config_dir`) and data (`_input_data_dir`)
 
-        input_config_dir (str): The directory where standard SageMaker configuration files are located,
-            e.g. /opt/ml/input/config/.
+        input_config_dir (str): The directory where standard SageMaker configuration files are
+                                located, e.g. /opt/ml/input/config/.
 
             SageMaker training creates the following files in this folder when training starts:
-                - `hyperparameters.json`: Amazon SageMaker makes the hyperparameters in a CreateTrainingJob
-                        request available in this file.
-                - `inputdataconfig.json`: You specify data channel information in the InputDataConfig
-                        parameter in a CreateTrainingJob request. Amazon SageMaker makes this information
-                        available in this file.
-                - `resourceconfig.json`: name of the current host and all host containers in the training
+                - `hyperparameters.json`: Amazon SageMaker makes the hyperparameters in a
+                                          CreateTrainingJob request available in this file.
+                - `inputdataconfig.json`: You specify data channel information in the
+                                          InputDataConfig parameter in a CreateTrainingJob request.
+                                          Amazon SageMaker makes this information available in this
+                                          file.
+                - `resourceconfig.json`: name of the current host and all host containers in the
+                                         training
 
             More information about these files can be find here:
                 https://docs.aws.amazon.com/sagemaker/latest/dg/your-algorithms-training-algo.html
 
         model_dir (str):  the directory where models should be saved, e.g., /opt/ml/model/
 
-        output_dir (str): The directory where training success/failure indications will be written, e.g.
-            /opt/ml/output. To save non-model artifacts check `output_data_dir`.
+        output_dir (str): The directory where training success/failure indications will be written,
+                          e.g. /opt/ml/output. To save non-model artifacts check `output_data_dir`.
 
-        hyperparameters (dict[string, object]): An instance of `HyperParameters` containing the training job
-                                                hyperparameters.
+        hyperparameters (dict[string, object]): An instance of `HyperParameters` containing the
+                                                training job hyperparameters.
 
-        resource_config (dict[string, object]): the contents from /opt/ml/input/config/resourceconfig.json.
+        resource_config (dict[string, object]): the contents from
+                                                /opt/ml/input/config/resourceconfig.json.
             It has the following keys:
                 - current_host: The name of the current container on the container network.
                     For example, 'algo-1'.
@@ -503,18 +512,24 @@ class TrainingEnv(_Env):
             You can find more information about /opt/ml/input/config/inputdataconfig.json here:
             https://docs.aws.amazon.com/sagemaker/latest/dg/your-algorithms-training-algo.html#your-algorithms-training-algo-running-container-inputdataconfig
 
-        output_data_dir (str): The dir to write non-model training artifacts (e.g. evaluation results) which will be
-            retained by SageMaker, e.g. /opt/ml/output/data. As your algorithm runs in a container, it generates
-            output including the status of the training job and model and output artifacts. Your algorithm should
-            write this information to the this directory.
+        output_data_dir (str): The dir to write non-model training artifacts (e.g. evaluation
+                               results) which will be retained by SageMaker,
+                               e.g. /opt/ml/output/data. As your algorithm runs in a container,
+                               it generates output including the status of the training job and
+                               model and output artifacts. Your algorithm should write this
+                               information to the this directory.
 
-        hosts (list[str]): The list of names of all containers on the container network, sorted lexicographically.
-            For example, `['algo-1', 'algo-2', 'algo-3']` for a three-node cluster.
+        hosts (list[str]): The list of names of all containers on the container network, sorted
+                           lexicographically. For example, `['algo-1', 'algo-2', 'algo-3']` for
+                           a three-node cluster.
 
-        channel_input_dirs (dict[string, string]): containing the data channels and the directories where the
-            training data was saved. When you run training, you can partition your training data into different logical
-            'channels'. Depending on your problem, some common channel ideas are: 'train', 'test',
-            'evaluation' or 'images','labels'.
+        channel_input_dirs (dict[string, string]): containing the data channels and the directories
+                                                   where the training data was saved. When you run
+                                                   training, you can partition your training data
+                                                   into different logical 'channels'. Depending on
+                                                   your problem, some common channel ideas are:
+                                                   'train', 'test', 'evaluation' or 'images',
+                                                   'labels'.
 
             The format of channel_input_dir is as follows:
                 - `channel`(str) - the name of the channel defined in the input_data_config.
@@ -532,29 +547,40 @@ class TrainingEnv(_Env):
         super(TrainingEnv, self).__init__()
 
         resource_config = resource_config or read_resource_config()
-        current_host = resource_config['current_host']
-        hosts = resource_config['hosts']
-        network_interface_name = resource_config.get('network_interface_name', 'ethwe')
+        current_host = resource_config["current_host"]
+        hosts = resource_config["hosts"]
         input_data_config = input_data_config or read_input_data_config()
 
         all_hyperparameters = hyperparameters or read_hyperparameters()
-        split_result = _mapping.split_by_criteria(all_hyperparameters, keys=_params.SAGEMAKER_HYPERPARAMETERS,
-                                                  prefix=_params.SAGEMAKER_PREFIX)
+        split_result = _mapping.split_by_criteria(
+            all_hyperparameters,
+            keys=_params.SAGEMAKER_HYPERPARAMETERS,
+            prefix=_params.SAGEMAKER_PREFIX,
+        )
 
         sagemaker_hyperparameters = split_result.included
         additional_framework_parameters = {
-            k: sagemaker_hyperparameters[k] for k in sagemaker_hyperparameters.keys()
+            k: sagemaker_hyperparameters[k]
+            for k in sagemaker_hyperparameters.keys()
             if k not in _params.SAGEMAKER_HYPERPARAMETERS
         }
 
-        sagemaker_region = sagemaker_hyperparameters.get(_params.REGION_NAME_PARAM, boto3.session.Session().region_name)
+        sagemaker_region = sagemaker_hyperparameters.get(
+            _params.REGION_NAME_PARAM, boto3.session.Session().region_name
+        )
 
-        os.environ[_params.JOB_NAME_ENV] = sagemaker_hyperparameters.get(_params.JOB_NAME_PARAM, '')
+        os.environ[_params.JOB_NAME_ENV] = sagemaker_hyperparameters.get(_params.JOB_NAME_PARAM, "")
         os.environ[_params.CURRENT_HOST_ENV] = current_host
-        os.environ[_params.REGION_NAME_ENV] = sagemaker_region or ''
+        os.environ[_params.REGION_NAME_ENV] = sagemaker_region or ""
 
         self._hosts = hosts
-        self._network_interface_name = network_interface_name
+
+        # eth0 is the default network interface defined by SageMaker with VPC support and
+        # local mode.
+        # ethwe is the current network interface defined by SageMaker training, it will be
+        # changed to eth0 in the short future.
+        self._network_interface_name = resource_config.get("network_interface_name", "eth0")
+
         self._hyperparameters = split_result.excluded
         self._additional_framework_parameters = additional_framework_parameters
         self._resource_config = resource_config
@@ -567,12 +593,15 @@ class TrainingEnv(_Env):
         # override base class attributes
         if self._module_name is None:
             self._module_name = str(sagemaker_hyperparameters.get(_params.USER_PROGRAM_PARAM, None))
-        self._user_entry_point = self._user_entry_point or sagemaker_hyperparameters.get(_params.USER_PROGRAM_PARAM)
+        self._user_entry_point = self._user_entry_point or sagemaker_hyperparameters.get(
+            _params.USER_PROGRAM_PARAM
+        )
 
         self._module_dir = str(sagemaker_hyperparameters.get(_params.SUBMIT_DIR_PARAM, code_dir))
         self._log_level = sagemaker_hyperparameters.get(_params.LOG_LEVEL_PARAM, logging.INFO)
-        self._sagemaker_s3_output = sagemaker_hyperparameters.get(_params.S3_OUTPUT_LOCATION_PARAM,
-                                                                  None)
+        self._sagemaker_s3_output = sagemaker_hyperparameters.get(
+            _params.S3_OUTPUT_LOCATION_PARAM, None
+        )
         self._framework_module = os.environ.get(_params.FRAMEWORK_TRAINING_MODULE_ENV, None)
 
         self._input_dir = input_dir
@@ -580,6 +609,21 @@ class TrainingEnv(_Env):
         self._output_dir = output_dir
         self._job_name = os.environ.get(_params.TRAINING_JOB_ENV.upper(), None)
 
+        self._master_hostname = list(hosts)[0]
+        self._is_master = current_host == self._master_hostname
+
+    @property
+    def is_master(self):  # type: () -> bool
+        """Returns True if host is master
+        """
+        return self._is_master
+
+    @property
+    def master_hostname(self):  # type: () -> str
+        """Returns the hostname of the master node
+        """
+        return self._master_hostname
+
     @property
     def job_name(self):  # type: () -> str
         """The name of the current training job.
@@ -591,13 +635,13 @@ class TrainingEnv(_Env):
 
     @property
     def additional_framework_parameters(self):  # type: () -> dict
-        """The dict of additional framework hyperparameters. All the hyperparameters prefixed with 'sagemaker_' but
-            not in SAGEMAKER_HYPERPARAMETERS will be included here.
+        """The dict of additional framework hyperparameters. All the hyperparameters prefixed with
+        'sagemaker_' but not in SAGEMAKER_HYPERPARAMETERS will be included here.
 
         Returns:
-            dict: additional framework hyperparameters, SageMaker Python SDK adds hyperparameters with a prefix
-            **sagemaker_** during training. These hyperparameters are framework independent settings and are not
-            defined by the user.
+            dict: additional framework hyperparameters, SageMaker Python SDK adds hyperparameters
+                  with a prefix **sagemaker_** during training. These hyperparameters are
+                  framework independent settings and are not defined by the user.
         """
         return self._additional_framework_parameters
 
@@ -625,24 +669,36 @@ class TrainingEnv(_Env):
         """
 
         env = {
-            'hosts':            self.hosts, 'network_interface_name': self.network_interface_name,
-            'hps':              self.hyperparameters, 'user_entry_point': self.user_entry_point,
-            'framework_params': self.additional_framework_parameters,
-            'resource_config':  self.resource_config, 'input_data_config': self.input_data_config,
-            'output_data_dir':  self.output_data_dir, 'channels': sorted(self.channel_input_dirs.keys()),
-            'current_host':     self.current_host, 'module_name': self.module_name, 'log_level': self.log_level,
-            'framework_module': self.framework_module, 'input_dir': self.input_dir,
-            'input_config_dir': self.input_config_dir, 'output_dir': self.output_dir, 'num_cpus': self.num_cpus,
-            'num_gpus':         self.num_gpus, 'model_dir': self.model_dir, 'module_dir': self.module_dir,
-            'training_env':     dict(self), 'user_args': self.to_cmd_args(),
-            'output_intermediate_dir': self.output_intermediate_dir
+            "hosts": self.hosts,
+            "network_interface_name": self.network_interface_name,
+            "hps": self.hyperparameters,
+            "user_entry_point": self.user_entry_point,
+            "framework_params": self.additional_framework_parameters,
+            "resource_config": self.resource_config,
+            "input_data_config": self.input_data_config,
+            "output_data_dir": self.output_data_dir,
+            "channels": sorted(self.channel_input_dirs.keys()),
+            "current_host": self.current_host,
+            "module_name": self.module_name,
+            "log_level": self.log_level,
+            "framework_module": self.framework_module,
+            "input_dir": self.input_dir,
+            "input_config_dir": self.input_config_dir,
+            "output_dir": self.output_dir,
+            "num_cpus": self.num_cpus,
+            "num_gpus": self.num_gpus,
+            "model_dir": self.model_dir,
+            "module_dir": self.module_dir,
+            "training_env": dict(self),
+            "user_args": self.to_cmd_args(),
+            "output_intermediate_dir": self.output_intermediate_dir,
         }
 
         for name, path in self.channel_input_dirs.items():
-            env['channel_%s' % name] = path
+            env["channel_%s" % name] = path
 
         for key, value in self.hyperparameters.items():
-            env['hp_%s' % key] = value
+            env["hp_%s" % key] = value
 
         return _mapping.to_env_vars(env)
 
@@ -659,12 +715,14 @@ class TrainingEnv(_Env):
     def channel_input_dirs(self):  # type: () -> dict
         """A dict[str, str] containing the data channels and the directories where the training
         data was saved.
-        When you run training, you can partition your training data into different logical "channels".
+        When you run training, you can partition your training data into different logical
+        "channels".
         Depending on your problem, some common channel ideas are: "train", "test", "evaluation"
             or "images',"labels".
         The format of channel_input_dir is as follows:
             - `channel`[key](str) - the name of the channel defined in the input_data_config.
-            - `training data path`[value](str) - the path to the directory where the training data is saved.
+            - `training data path`[value](str) - the path to the directory where the training
+                                                 data is saved.
         Returns:
             dict[str, str] with the information about the channels.
         """
@@ -691,15 +749,17 @@ class TrainingEnv(_Env):
 
     @property
     def input_config_dir(self):  # type: () -> str
-        """The directory where standard SageMaker configuration files are located, e.g. /opt/ml/input/config/.
+        """The directory where standard SageMaker configuration files are located, e.g.
+        /opt/ml/input/config/.
         SageMaker training creates the following files in this folder when training starts:
-            - `hyperparameters.json`: Amazon SageMaker makes the hyperparameters in a CreateTrainingJob
-                request available in this file.
-            - `inputdataconfig.json`: You specify data channel information in the InputDataConfig parameter
-                in a CreateTrainingJob request. Amazon SageMaker makes this information available
-                in this file.
-            - `resourceconfig.json`: name of the current host and all host containers in the training
-        More information about this files can be find here:
+            - `hyperparameters.json`: Amazon SageMaker makes the hyperparameters in a
+                                      CreateTrainingJob request available in this file.
+            - `inputdataconfig.json`: You specify data channel information in the
+                                      InputDataConfig parameter in a CreateTrainingJob request.
+                                      Amazon SageMaker makes this information available in this
+                                      file.
+            - `resourceconfig.json`: name of the current host and all host containers in the
+                                     training More information about this files can be find here:
             https://docs.aws.amazon.com/sagemaker/latest/dg/your-algorithms-training-algo.html
         Returns:
             str: the path of the input directory, e.g. /opt/ml/input/config/
@@ -708,7 +768,8 @@ class TrainingEnv(_Env):
 
     @property
     def output_dir(self):  # type: () -> str
-        """The directory where training success/failure indications will be written, e.g. /opt/ml/output.
+        """The directory where training success/failure indications will be written,
+        e.g. /opt/ml/output.
         To save non-model artifacts check `output_data_dir`.
         Returns:
             str: the path to the output directory, e.g. /opt/ml/output/.
@@ -728,10 +789,11 @@ class TrainingEnv(_Env):
     def resource_config(self):  # type: () -> dict
         """A dictionary with the contents from /opt/ml/input/config/resourceconfig.json.
                 It has the following keys:
-                    - current_host: The name of the current container on the container network.
-                        For example, 'algo-1'.
-                    -  hosts: The list of names of all containers on the container network, sorted lexicographically.
-                        For example, `["algo-1", "algo-2", "algo-3"]` for a three-node cluster.
+                    - current_host: The name of the current container on the container
+                                    network. For example, 'algo-1'.
+                    -  hosts: The list of names of all containers on the container network,
+                              sorted lexicographically. For example,
+                              `["algo-1", "algo-2", "algo-3"]` for a three-node cluster.
                 Returns:
                     dict[str, str or list(str)]
         """
@@ -740,8 +802,8 @@ class TrainingEnv(_Env):
     @property
     def input_data_config(self):  # type: () -> dict
         """A dictionary with the contents from /opt/ml/input/config/inputdataconfig.json.
-                For example, suppose that you specify three data channels (train, evaluation, and validation) in
-                your request. This dictionary will contain:
+                For example, suppose that you specify three data channels (train,
+                evaluation, and validation) in your request. This dictionary will contain:
                 ```{"train": {
                         "ContentType":  "trainingContentType",
                         "TrainingInputMode": "File",
@@ -769,8 +831,8 @@ class TrainingEnv(_Env):
 
     @property
     def output_data_dir(self):  # type: () -> str
-        """The dir to write non-model training artifacts (e.g. evaluation results) which will be retained
-        by SageMaker, e.g. /opt/ml/output/data/{current_host}.
+        """The dir to write non-model training artifacts (e.g. evaluation results) which will be
+        retained by SageMaker, e.g. /opt/ml/output/data/{current_host}.
         As your algorithm runs in a container, it generates output including the status of the
         training job and model and output artifacts. Your algorithm should write this information
         to the this directory.
@@ -798,10 +860,11 @@ class TrainingEnv(_Env):
 
 
 class ServingEnv(_Env):
-    """Provides access to aspects of the serving environment relevant to serving containers, including
-       system characteristics, environment variables and configuration settings.
+    """Provides access to aspects of the serving environment relevant to serving containers,
+    including system characteristics, environment variables and configuration settings.
 
-       The ServingEnv is a read-only snapshot of the container environment. It does not contain any form of state.
+       The ServingEnv is a read-only snapshot of the container environment. It does not contain any
+       form of state.
 
        It is a dictionary like object, allowing any builtin function that works with dictionary.
 
@@ -823,8 +886,8 @@ class ServingEnv(_Env):
                 as specified in the user-supplied SAGEMAKER_DEFAULT_INVOCATIONS_ACCEPT environment
                 variable. Otherwise, returns 'application/json' by default.
                 For example: application/json
-            http_port (str): Port that SageMaker will use to handle invocations and pings against the
-                running Docker container. Default is 8080. For example: 8080
+            http_port (str): Port that SageMaker will use to handle invocations and pings against
+                             the running Docker container. Default is 8080. For example: 8080
             safe_port_range (str): HTTP port range that can be used by customers to avoid collisions
                 with the HTTP port specified by SageMaker for handling pings and invocations.
                 For example: 1111-2222
@@ -833,12 +896,12 @@ class ServingEnv(_Env):
     def __init__(self):
         super(ServingEnv, self).__init__()
 
-        use_nginx = util.strtobool(os.environ.get(_params.USE_NGINX_ENV, 'true')) == 1
-        model_server_timeout = int(os.environ.get(_params.MODEL_SERVER_TIMEOUT_ENV, '60'))
+        use_nginx = util.strtobool(os.environ.get(_params.USE_NGINX_ENV, "true")) == 1
+        model_server_timeout = int(os.environ.get(_params.MODEL_SERVER_TIMEOUT_ENV, "60"))
         model_server_workers = int(os.environ.get(_params.MODEL_SERVER_WORKERS_ENV, num_cpus()))
         framework_module = os.environ.get(_params.FRAMEWORK_SERVING_MODULE_ENV, None)
         default_accept = os.environ.get(_params.DEFAULT_INVOCATIONS_ACCEPT_ENV, _content_types.JSON)
-        http_port = os.environ.get(_params.SAGEMAKER_BIND_TO_PORT_ENV, '8080')
+        http_port = os.environ.get(_params.SAGEMAKER_BIND_TO_PORT_ENV, "8080")
         safe_port_range = os.environ.get(_params.SAGEMAKER_SAFE_PORT_RANGE_ENV)
 
         self._use_nginx = use_nginx
@@ -858,8 +921,11 @@ class ServingEnv(_Env):
     @property
     def model_server_timeout(self):  # type: () -> int
         """Returns:
-            int: Timeout in seconds for the model server. This is passed over to gunicorn, from the docs:
-                Workers silent for more than this many seconds are killed and restarted. Our default value is 60."""
+            int: Timeout in seconds for the model server. This is passed over to gunicorn,
+                 from the docs:
+                 Workers silent for more than this many seconds are killed and restarted.
+                 Our default value is 60. If ``use_nginx`` is True, then this same value
+                 will be used for nginx's proxy_read_timeout."""
         return self._model_server_timeout
 
     @property
@@ -878,22 +944,24 @@ class ServingEnv(_Env):
     @property
     def default_accept(self):  # type: () -> str
         """Returns:
-            str: The desired MIME type of the inference in the response. For example: application/json.
+            str: The desired MIME type of the inference in the response. For example:
+                 application/json.
                 Default: application/json"""
         return self._default_accept
 
     @property
     def http_port(self):  # type: () -> str
         """Returns:
-            str: HTTP port that SageMaker will use to handle invocations and pings against the running
-                Docker container. Default is 8080. For example: 8080"""
+            str: HTTP port that SageMaker will use to handle invocations and pings against
+                 the running Docker container. Default is 8080. For example: 8080"""
         return self._http_port
 
     @property
     def safe_port_range(self):  # type: () -> str
         """Returns:
-            str: HTTP port range that can be used by customers to avoid collisions with the HTTP port
-                specified by SageMaker for handling pings and invocations. For example: 1111-2222"""
+            str: HTTP port range that can be used by customers to avoid collisions with the
+                 HTTP port specified by SageMaker for handling pings and invocations.
+                 For example: 1111-2222"""
         return self._safe_port_range
 
 
@@ -907,7 +975,7 @@ def write_env_vars(env_vars=None):  # type: (dict) -> None
 
     """
     env_vars = env_vars or {}
-    env_vars['PYTHONPATH'] = ':'.join(sys.path)
+    env_vars["PYTHONPATH"] = ":".join(sys.path)
 
     for name, value in env_vars.items():
         os.environ[name] = value
